==> git-11.0.67.tgz: package/src/_/doCommitPushPublish.ts <==
import { getDevOpsCore } from '@thatdevops/core/devopsCore'
import getPkg from '@thatdevops/core/pkgman/getPkg'
import getProjectPkgs from '@thatdevops/core/pkgman/getProjectPkgs'
import Pkg from '@thatdevops/core/pkgman/Pkg'
import writePkg from '@thatdevops/core/pkgman/writePkg'
import echo from '@thatdevops/core/utils/echo'
import spawn from '@thatdevops/core/utils/spawn'
import doNothing from '@thats/utils/misc/doNothing'
import chunk from 'lodash/chunk'
import { DateTime } from 'luxon'
import path from 'path'
import doFindChangedPackages from './doFindChangedPackages'
import { doPublishPackage } from './doPublishPackage'
import { BumpType, versionBump } from './versionBump'

export async function doCommitPushPublish(props?: { bumpType?: BumpType; commitMessage?: string }) {
	const { rootPath, parallel = 0 } = getDevOpsCore()
	const { commitMessage = `Publish ${DateTime.now().toISODate()}`, bumpType = BumpType.patch } =
		props ?? {}
	const { publishable } = getDevOpsCore()
	if (!publishable) {
		echo('Publishing not allowed (set the dev ops "publishable" flag')
		return
	}

	echo(`Committing "Pre ${commitMessage}"`)
	await spawn('git', ['add', '-A']).catch(doNothing)
	await spawn('git', ['commit', '-a', '-m', `Pre ${commitMessage}`]).catch(doNothing)

	// Find packages that have changes since their last publish
	echo('Finding changed packages')
	let changedPkgs = await doFindChangedPackages()
	const effectedPkgs = await doFindEffectedPackages(changedPkgs)

	// Calculate new versions for changed packages
	const newVersions = doCalcNewVersions(effectedPkgs, bumpType)

	// Update any dependencies that have changed
	echo('Updating related dependencies')
	await doUpdateDependencies(effectedPkgs, newVersions)

	echo('Publishing effected packages')
	if (!!parallel) {
		const chunks = chunk(effectedPkgs, parallel)
		for (const chunki of chunks) {
			await Promise.all(chunki.map(c => doPublishPackage(c, newVersions[c.name])))
		}
	} else {
		for (const pkg of effectedPkgs) {
			await doPublishPackage(pkg, newVersions[pkg.name])
		}
	}

	echo('Updating DevOps package')
	const devopsPkg = getPkg(path.join(rootPath, 'devops'))
	await doUpdateDependencies([devopsPkg], newVersions)

	echo(`Committing "${commitMessage}"`)
	await spawn('git', ['add', '-A']).catch(doNothing)
	await spawn('git', ['commit', '-a', '-m', commitMessage]).catch(doNothing)

	echo('Pushing')
	await spawn('git', ['push']).catch(doNothing)

	echo('Setting publish dates')

	const rootPkg = getPkg(rootPath)
	rootPkg.json.devops = rootPkg.json.devops || {}
	rootPkg.json.devops.publishdates = rootPkg.json.devops.publishdates || {}
	const now = DateTime.now().plus({ minute: 1 }).toISO()
	for (const pkg of effectedPkgs) {
		rootPkg.json.devops.publishdates[pkg.json.name] = now
	}
	writePkg(rootPkg)
}

export default doCommitPushPublish

function doCalcNewVersions(pkgs: Pkg[], bumpType: BumpType = BumpType.patch) {
	const newVersions = {}
	for (const pkg of pkgs) {
		newVersions[pkg.name] = versionBump(pkg.version, bumpType)
	}
	return newVersions
}

async function doUpdateDependencies(pkgs: Pkg[], newVersions: Record<string, string>) {
	for (const pkg of pkgs) {
		for (const depList of ['dependencies', 'devDependencies']) {
			const deps = pkg.json[depList]
			for (const depName in deps) {
				if (newVersions[depName]) {
					const oldVersion = deps[depName]
					const newVersion = newVersions[depName]
					if (oldVersion !== newVersion) {
						deps[depName] = newVersion
					}
				}
			}
		}
		delete pkg.json.gitHead // Tidying up crap
		writePkg(pkg)
	}
}

async function doFindEffectedPackages(changedPkgs: Pkg[]): Promise<Pkg[]> {
	const allEffected = []
	let effected = await doFindNewEffectedPackages(changedPkgs)
	allEffected.push(...effected)
	while (effected.length) {
		const newEffected = await doFindNewEffectedPackages(effected)
		effected = newEffected.filter(p => !allEffected.find(p2 => p2.name === p.name))
		allEffected.push(...effected)
	}
	return allEffected
}

async function doFindNewEffectedPackages(changedPkgs: Pkg[]): Promise<Pkg[]> {
	const pkgs = await getProjectPkgs()
	const effected = []
	for (const changedPkg of changedPkgs) {
		effected.push(changedPkg)
		for (const pkg of pkgs) {
			if (!effected.find(p => p.name === pkg.name)) {
				for (const depList of ['dependencies', 'devDependencies']) {
					const deps = pkg.json[depList]
					for (const depName in deps) {
						if (depName === changedPkg.name) {
							effected.push(pkg)
						}
					}
				}
			}
		}
	}
	return effected
}

==> git-11.0.68.tgz: package/src/_/doCommitPushPublish.ts <==
import { getDevOpsCore } from '@thatdevops/core/devopsCore'
import getPkg from '@thatdevops/core/pkgman/getPkg'
import getProjectPkgs from '@thatdevops/core/pkgman/getProjectPkgs'
import Pkg from '@thatdevops/core/pkgman/Pkg'
import writePkg from '@thatdevops/core/pkgman/writePkg'
import echo from '@thatdevops/core/utils/echo'
import spawn from '@thatdevops/core/utils/spawn'
import doNothing from '@thats/utils/misc/doNothing'
import chunk from 'lodash/chunk'
import { DateTime } from 'luxon'
import path from 'path'
import doFindChangedPackages from './doFindChangedPackages'
import { doPublishPackage } from './doPublishPackage'
import { BumpType, versionBump } from './versionBump'

export async function doCommitPushPublish(props?: {
	force?: boolean
	bumpType?: BumpType
	commitMessage?: string
}) {
	const { rootPath, parallel = 0 } = getDevOpsCore()
	const {
		force = false,
		commitMessage = `Publish ${DateTime.now().toISODate()}`,
		bumpType = BumpType.patch,
	} = props ?? {}
	const { publishable } = getDevOpsCore()
	if (!publishable) {
		echo('Publishing not allowed (set the dev ops "publishable" flag')
		return
	}

	echo(`Committing "Pre ${commitMessage}"`)
	await spawn('git', ['add', '-A']).catch(doNothing)
	await spawn('git', ['commit', '-a', '-m', `Pre ${commitMessage}`]).catch(doNothing)

	// Find packages that have changes since their last publish
	echo('Finding changed packages')
	let changedPkgs = force ? await getProjectPkgs() : await doFindChangedPackages()

	if (!changedPkgs.length) {
		echo('No changes to publish')
		return
	}

	const effectedPkgs = await doFindEffectedPackages(changedPkgs)

	// Calculate new versions for changed packages
	const newVersions = doCalcNewVersions(effectedPkgs, bumpType)

	// Update any dependencies that have changed
	echo('Updating related dependencies')
	await doUpdateDependencies(effectedPkgs, newVersions)

	echo('Publishing effected packages')
	if (!!parallel) {
		const chunks = chunk(effectedPkgs, parallel)
		for (const chunki of chunks) {
			await Promise.all(chunki.map(c => doPublishPackage(c, newVersions[c.name])))
		}
	} else {
		for (const pkg of effectedPkgs) {
			await doPublishPackage(pkg, newVersions[pkg.name])
		}
	}

	echo('Updating DevOps package')
	const devopsPkg = getPkg(path.join(rootPath, 'devops'))
	await doUpdateDependencies([devopsPkg], newVersions)

	echo(`Committing "${commitMessage}"`)
	await spawn('git', ['add', '-A']).catch(doNothing)
	await spawn('git', ['commit', '-a', '-m', commitMessage]).catch(doNothing)

	echo('Pushing')
	await spawn('git', ['push']).catch(doNothing)

	echo('Setting publish dates')

	const rootPkg = getPkg(rootPath)
	rootPkg.json.devops = rootPkg.json.devops || {}
	rootPkg.json.devops.publishdates = rootPkg.json.devops.publishdates || {}
	const now = DateTime.now().plus({ minute: 1 }).toISO()
	for (const pkg of effectedPkgs) {
		rootPkg.json.devops.publishdates[pkg.json.name] = now
	}
	writePkg(rootPkg)
}

export default doCommitPushPublish

function doCalcNewVersions(pkgs: Pkg[], bumpType: BumpType = BumpType.patch) {
	const newVersions = {}
	for (const pkg of pkgs) {
		newVersions[pkg.name] = versionBump(pkg.version, bumpType)
	}
	return newVersions
}

async function doUpdateDependencies(pkgs: Pkg[], newVersions: Record<string, string>) {
	for (const pkg of pkgs) {
		for (const depList of ['dependencies', 'devDependencies']) {
			const deps = pkg.json[depList]
			for (const depName in deps) {
				if (newVersions[depName]) {
					const oldVersion = deps[depName]
					const newVersion = newVersions[depName]
					if (oldVersion !== newVersion) {
						deps[depName] = newVersion
					}
				}
			}
		}
		delete pkg.json.gitHead // Tidying up crap
		writePkg(pkg)
	}
}

async function doFindEffectedPackages(changedPkgs: Pkg[]): Promise<Pkg[]> {
	const allEffected = []
	let effected = await doFindNewEffectedPackages(changedPkgs)
	allEffected.push(...effected)
	while (effected.length) {
		const newEffected = await doFindNewEffectedPackages(effected)
		effected = newEffected.filter(p => !allEffected.find(p2 => p2.name === p.name))
		allEffected.push(...effected)
	}
	return allEffected
}

async function doFindNewEffectedPackages(changedPkgs: Pkg[]): Promise<Pkg[]> {
	const pkgs = await getProjectPkgs()
	const effected = []
	for (const changedPkg of changedPkgs) {
		effected.push(changedPkg)
		for (const pkg of pkgs) {
			if (!effected.find(p => p.name === pkg.name)) {
				for (const depList of ['dependencies', 'devDependencies']) {
					const deps = pkg.json[depList]
					for (const depName in deps) {
						if (depName === changedPkg.name) {
							effected.push(pkg)
						}
					}
				}
			}
		}
	}
	return effected
}
